# Generated by Django 3.1 on 2020-10-02 20:35

from django.db import migrations
import docrootcms.models


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='markdownpost',
            name='content',
            field=docrootcms.models.ContentMarkdownField(help_text='Markdown Reference: <a href="https://commonmark.org/help/">https://commonmark.org/help/</a>'),
        ),
        migrations.AlterField(
            model_name='markdownpost',
            name='teaser',
            field=docrootcms.models.ContentMarkdownField(blank=True, help_text='Markdown Reference: <a href="https://commonmark.org/help/">https://commonmark.org/help/</a>', null=True),
        ),
    ]
