{% extends "admin/base_site.html" %}

{# adding bootstrap and js #}
{% block extrastyle %}
    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.5.2/css/bootstrap.min.css" integrity="sha384-JcKb8q3iqJ61gNV9KGb8thSsNjpSL0n8PARn9HuZOnIxN0hoP+VmmDGMN5t9UJ0Z" crossorigin="anonymous">
    <style>
        /* Bootstrap issues with admin */
        a.deletelink, ::after, ::before {
            box-sizing: revert;
        }
        div#content h1 {font-size: 2rem; font-weight: 400;}
        #searchbar {height: auto !important;}
        div.module caption { caption-side: top !important; }
        .select2-input {min-width: 150px;}
        .select2-container {width: 100%;}
        .select2-choices {width: 100% !important;}
        .collapse { display: block !important; }
    </style>

    <script src="https://code.jquery.com/jquery-3.5.1.min.js" integrity="sha256-9/aliU8dGd2tb6OSsuzixeV4y/faTqgFtohetphbbj0=" crossorigin="anonymous"></script>
    <script src="https://stackpath.bootstrapcdn.com/bootstrap/4.5.2/js/bootstrap.bundle.min.js" integrity="sha384-LtrjvnR4Twt/qOuYxE721u19sVFLVSA4hf/rRt6PrZTmiPltdZcI7q7PXQBYTKyf" crossorigin="anonymous"></script>
{% endblock %}